@@ -11,20 +11,20 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+# include <config.h>
 #endif
 
 #ifdef HAVE_STRING_H
-#include <string.h>
+# include <string.h>
 #else
-#include <strings.h>
-#ifndef strchr
-#define strchr index
-#endif
+# include <strings.h>
+# ifndef strchr
+#  define strchr index
+# endif
 #endif
 
 /* Return the length of the maximum inital segment of S
