@@ -1,5 +1,5 @@
 /* pathchk -- check whether pathnames are valid or portable
-   Copyright (C) 91, 92, 93, 1994 Free Software Foundation, Inc.
+   Copyright (C) 91, 92, 93, 94, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Usage: pathchk [-p] [--portability] path...
 
@@ -43,9 +43,8 @@
 #include <getopt.h>
 #include <sys/types.h>
 
-#include "version.h"
 #include "system.h"
-#include "safe-stat.h"
+#include "error.h"
 
 #ifdef _POSIX_VERSION
 #include <limits.h>
@@ -92,18 +91,17 @@
 #endif
 
 char *xstrdup ();
-void error ();
 
-static int validate_path ();
-static void usage ();
+static int validate_path __P ((char *path, int portability));
+static void usage __P ((int status));
 
 /* The name this program was run with. */
 char *program_name;
 
-/* If non-zero, display usage information and exit.  */
+/* If nonzero, display usage information and exit.  */
 static int show_help;
 
-/* If non-zero, print the version on standard output and exit.  */
+/* If nonzero, print the version on standard output and exit.  */
 static int show_version;
 
 static struct option const longopts[] =
@@ -114,16 +112,17 @@ static struct option const longopts[] =
   {NULL, 0, NULL, 0}
 };
 
-void
-main (argc, argv)
-     int argc;
-     char **argv;
+int
+main (int argc, char **argv)
 {
   int exit_status = 0;
   int check_portability = 0;
   int optc;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 
   while ((optc = getopt_long (argc, argv, "p", longopts, (int *) 0)) != EOF)
     {
@@ -143,7 +142,7 @@ main (argc, argv)
 
   if (show_version)
     {
-      printf ("pathchk - %s\n", version_string);
+      printf ("pathchk (%s) %s\n", GNU_PACKAGE, VERSION);
       exit (0);
     }
 
@@ -152,7 +151,7 @@ main (argc, argv)
 
   if (optind == argc)
     {
-      error (0, 0, "too few arguments");
+      error (0, 0, _("too few arguments"));
       usage (1);
     }
 
@@ -175,28 +174,27 @@ static char const portable_chars[256] =
   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, /* 80-95 */
   0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, /* 96-111 */
   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, /* 112-127 */
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
 };
 
 /* If PATH contains only portable characters, return 1, else 0.  */
 
 static int
-portable_chars_only (path)
-     const char *path;
+portable_chars_only (const char *path)
 {
   const char *p;
 
   for (p = path; *p; ++p)
     if (portable_chars[(const unsigned char) *p] == 0)
       {
-	error (0, 0, "path `%s' contains nonportable character `%c'",
+	error (0, 0, _("path `%s' contains nonportable character `%c'"),
 	       path, *p);
 	return 0;
       }
@@ -207,17 +205,16 @@ portable_chars_only (path)
    2 if it doesn't exist.  */
 
 static int
-dir_ok (path)
-     const char *path;
+dir_ok (const char *path)
 {
   struct stat stats;
 
-  if (SAFE_STAT (path, &stats))
+  if (stat (path, &stats))
     return 2;
 
   if (!S_ISDIR (stats.st_mode))
     {
-      error (0, 0, "`%s' is not a directory", path);
+      error (0, 0, _("`%s' is not a directory"), path);
       return 0;
     }
 
@@ -228,7 +225,7 @@ dir_ok (path)
   if (access (path, X_OK) != 0)
     {
       if (errno == EACCES)
-	error (0, 0, "directory `%s' is not searchable", path);
+	error (0, 0, _("directory `%s' is not searchable"), path);
       else
 	error (0, errno, "%s", path);
       return 0;
@@ -252,9 +249,7 @@ dir_ok (path)
    Return 0 if all of these tests are successful, 1 if any fail. */
 
 static int
-validate_path (path, portability)
-     char *path;
-     int portability;
+validate_path (char *path, int portability)
 {
   int path_max;
   int last_elem;		/* Nonzero if checking last element of path. */
@@ -289,13 +284,13 @@ validate_path (path, portability)
       while (*slash == '/')
 	slash++;
       start = slash;
-      slash = index (slash, '/');
+      slash = strchr (slash, '/');
       if (slash != NULL)
 	*slash = '\0';
       else
 	{
 	  last_elem = 1;
-	  slash = index (start, '\0');
+	  slash = strchr (start, '\0');
 	}
 
       if (!last_elem)
@@ -322,7 +317,7 @@ validate_path (path, portability)
 	name_max = _POSIX_NAME_MAX;
       if (length > name_max)
 	{
-	  error (0, 0, "name `%s' has length %d; exceeds limit of %d",
+	  error (0, 0, _("name `%s' has length %d; exceeds limit of %d"),
 		 start, length, name_max);
 	  free (parent);
 	  return 1;
@@ -346,9 +341,9 @@ validate_path (path, portability)
   if (path_max < 0)
     path_max = _POSIX_PATH_MAX;
   free (parent);
-  if (strlen (path) > path_max)
+  if (strlen (path) > (size_t) path_max)
     {
-      error (0, 0, "path `%s' has length %d; exceeds limit of %d",
+      error (0, 0, _("path `%s' has length %d; exceeds limit of %d"),
 	     path, strlen (path), path_max);
       return 1;
     }
@@ -357,21 +352,22 @@ validate_path (path, portability)
 }
 
 static void
-usage (status)
-     int status;
+usage (int status)
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("Usage: %s [OPTION]... PATH...\n", program_name);
-      printf ("\
+      printf (_("Usage: %s [OPTION]... NAME...\n"), program_name);
+      printf (_("\
+Diagnose unportable constructs in NAME.\n\
 \n\
   -p, --portability   check for all POSIX systems, not only this one\n\
       --help          display this help and exit\n\
       --version       output version information and exit\n\
-");
+"));
+      puts (_("\nReport bugs to sh-utils-bugs@gnu.ai.mit.edu"));
     }
   exit (status);
 }
