@@ -1,5 +1,5 @@
 /* printf - format and print data
-   Copyright (C) 90, 91, 92, 93, 1994 Free Software Foundation, Inc.
+   Copyright (C) 90, 91, 92, 93, 94, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Usage: printf format [argument...]
 
@@ -49,8 +49,8 @@
 #include <getopt.h>
 
 #include "system.h"
-#include "version.h"
 #include "long-options.h"
+#include "error.h"
 
 #ifndef STDC_HEADERS
 double strtod ();
@@ -63,17 +63,16 @@ unsigned long strtoul ();
 #define octtobin(c) ((c) - '0')
 
 char *xmalloc ();
-void error ();
 
-static double xstrtod ();
-static int print_esc ();
-static int print_formatted ();
-static long xstrtol ();
-static unsigned long xstrtoul ();
-static void print_direc ();
-static void print_esc_char ();
-static void print_esc_string ();
-static void verify ();
+static double xstrtod __P ((char *s));
+static int print_esc __P ((char *escstart));
+static int print_formatted __P ((char *format, int argc, char **argv));
+static long xstrtol __P ((char *s));
+static unsigned long xstrtoul __P ((char *s));
+static void print_direc __P ((char *start, size_t length, int field_width, int precision, char *argument));
+static void print_esc_char __P ((int c));
+static void print_esc_string __P ((char *str));
+static void verify __P ((char *s, char *end));
 
 /* The value to return to the calling program.  */
 static int exit_status;
@@ -82,20 +81,20 @@ static int exit_status;
 char *program_name;
 
 static void
-usage (status)
-     int status;
+usage (int status)
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("\
+      printf (_("\
 Usage: %s FORMAT [ARGUMENT]...\n\
   or:  %s OPTION\n\
-",
+"),
 	      program_name, program_name);
-      printf ("\
+      printf (_("\
+Print ARGUMENT(s) according to FORMAT.\n\
 \n\
   --help      display this help and exit\n\
   --version   output version information and exit\n\
@@ -120,27 +119,32 @@ FORMAT controls the output as in C printf.  Interpreted sequences are:\n\
 \n\
 and all C format specifications ending with one of diouxXfeEgGcs, with\n\
 ARGUMENTs converted to proper type first.  Variable widths are handled.\n\
-");
+"));
+      puts (_("\nReport bugs to sh-utils-bugs@gnu.ai.mit.edu"));
     }
   exit (status);
 }
 
-void
-main (argc, argv)
-     int argc;
-     char **argv;
+int
+main (int argc, char **argv)
 {
   char *format;
   int args_used;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
   exit_status = 0;
 
-  parse_long_options (argc, argv, "printf", version_string, usage);
+  /* Don't recognize --help or --version if POSIXLY_CORRECT is set.  */
+  if (getenv ("POSIXLY_CORRECT") == NULL)
+    parse_long_options (argc, argv, "printf", GNU_PACKAGE, VERSION, usage);
 
   if (argc == 1)
     {
-      fprintf (stderr, "Usage: %s format [argument...]\n", program_name);
+      fprintf (stderr, _("Usage: %s format [argument...]\n"), program_name);
       exit (1);
     }
 
@@ -156,6 +160,9 @@ main (argc, argv)
     }
   while (args_used > 0 && argc > 0);
 
+  if (argc > 0)
+    error (0, 0, _("warning: excess arguments have been ignored"));
+
   exit (exit_status);
 }
 
@@ -164,10 +171,7 @@ main (argc, argv)
    Return the number of elements of ARGV used.  */
 
 static int
-print_formatted (format, argc, argv)
-     char *format;
-     int argc;
-     char **argv;
+print_formatted (char *format, int argc, char **argv)
 {
   int save_argc = argc;		/* Preserve original value.  */
   char *f;			/* Pointer into `format'.  */
@@ -199,7 +203,7 @@ print_formatted (format, argc, argv)
 		}
 	      break;
 	    }
-	  if (index ("-+ #", *f))
+	  if (strchr ("-+ #", *f))
 	    {
 	      ++f;
 	      ++direc_length;
@@ -252,8 +256,8 @@ print_formatted (format, argc, argv)
 	      ++f;
 	      ++direc_length;
 	    }
-	  if (!index ("diouxXfeEgGcs", *f))
-	    error (1, 0, "%%%c: invalid directive", *f);
+	  if (!strchr ("diouxXfeEgGcs", *f))
+	    error (1, 0, _("%%%c: invalid directive"), *f);
 	  ++direc_length;
 	  if (argc > 0)
 	    {
@@ -284,8 +288,7 @@ print_formatted (format, argc, argv)
    besides the backslash. */
 
 static int
-print_esc (escstart)
-     char *escstart;
+print_esc (char *escstart)
 {
   register char *p = escstart + 1;
   int esc_value = 0;		/* Value of \nnn escape. */
@@ -299,7 +302,7 @@ print_esc (escstart)
 	   ++esc_length, ++p)
 	esc_value = esc_value * 16 + hextobin (*p);
       if (esc_length == 0)
-	error (1, 0, "missing hexadecimal number in escape");
+	error (1, 0, _("missing hexadecimal number in escape"));
       putchar (esc_value);
     }
   else if (*p == '0')
@@ -310,18 +313,17 @@ print_esc (escstart)
 	esc_value = esc_value * 8 + octtobin (*p);
       putchar (esc_value);
     }
-  else if (index ("\"\\abcfnrtv", *p))
+  else if (strchr ("\"\\abcfnrtv", *p))
     print_esc_char (*p++);
   else
-    error (1, 0, "\\%c: invalid escape", *p);
+    error (1, 0, _("\\%c: invalid escape"), *p);
   return p - escstart - 1;
 }
 
 /* Output a single-character \ escape.  */
 
 static void
-print_esc_char (c)
-     char c;
+print_esc_char (int c)
 {
   switch (c)
     {
@@ -358,8 +360,7 @@ print_esc_char (c)
 /* Print string STR, evaluating \ escapes. */
 
 static void
-print_esc_string (str)
-     char *str;
+print_esc_string (char *str)
 {
   for (; *str; str++)
     if (*str == '\\')
@@ -374,12 +375,7 @@ print_esc_string (str)
    '*' values in those fields. */
 
 static void
-print_direc (start, length, field_width, precision, argument)
-     char *start;
-     size_t length;
-     int field_width;
-     int precision;
-     char *argument;
+print_direc (char *start, size_t length, int field_width, int precision, char *argument)
 {
   char *p;		/* Null-terminated copy of % directive. */
 
@@ -474,8 +470,7 @@ print_direc (start, length, field_width, precision, argument)
 }
 
 static unsigned long
-xstrtoul (s)
-     char *s;
+xstrtoul (char *s)
 {
   char *end;
   unsigned long val;
@@ -487,8 +482,7 @@ xstrtoul (s)
 }
 
 static long
-xstrtol (s)
-     char *s;
+xstrtol (char *s)
 {
   char *end;
   long val;
@@ -500,8 +494,7 @@ xstrtol (s)
 }
 
 static double
-xstrtod (s)
-     char *s;
+xstrtod (char *s)
 {
   char *end;
   double val;
@@ -513,8 +506,7 @@ xstrtod (s)
 }
 
 static void
-verify (s, end)
-     char *s, *end;
+verify (char *s, char *end)
 {
   if (errno)
     {
@@ -524,9 +516,9 @@ verify (s, end)
   else if (*end)
     {
       if (s == end)
-	error (0, 0, "%s: expected a numeric value", s);
+	error (0, 0, _("%s: expected a numeric value"), s);
       else
-	error (0, 0, "%s: value not completely converted", s);
+	error (0, 0, _("%s: value not completely converted"), s);
       exit_status = 1;
     }
 }
