@@ -1,5 +1,5 @@
 /* stty -- change and print terminal line settings
-   Copyright (C) 90, 91, 92, 93, 1994 Free Software Foundation, Inc.
+   Copyright (C) 90, 91, 92, 93, 94, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Usage: stty [-ag] [--all] [--save] [setting...]
 
@@ -28,132 +28,132 @@
    David MacKenzie <djm@gnu.ai.mit.edu> */
 
 #include <config.h>
+
+#ifdef TERMIOS_NEEDS_XOPEN_SOURCE
+# define _XOPEN_SOURCE
+#endif
+
 #include <stdio.h>
 #include <sys/types.h>
 #include <termios.h>
 #ifdef GWINSZ_IN_SYS_IOCTL
-#include <sys/ioctl.h>
+# include <sys/ioctl.h>
 #endif
 #ifdef WINSIZE_IN_PTEM
-#include <sys/stream.h>
-#include <sys/ptem.h>
+# include <sys/stream.h>
+# include <sys/ptem.h>
+#endif
+#ifdef GWINSZ_IN_SYS_PTY
+# include <sys/ioctl.h>
+# include <sys/tty.h>
+# include <sys/pty.h>
 #endif
 #include <getopt.h>
-#ifdef __STDC__
-#include <stdarg.h>
-#define VA_START(args, lastarg) va_start(args, lastarg)
+#if PROTOTYPES
+# include <stdarg.h>
+# define VA_START(args, lastarg) va_start(args, lastarg)
 #else
-#include <varargs.h>
-#define VA_START(args, lastarg) va_start(args)
+# include <varargs.h>
+# define VA_START(args, lastarg) va_start(args)
 #endif
 
 #include "system.h"
-#include "version.h"
 #include "long-options.h"
-
-#if defined(GWINSZ_BROKEN)	/* Such as for SCO UNIX 3.2.2. */
-#undef TIOCGWINSZ
-#endif
+#include "error.h"
+#include "xstrtol.h"
 
 #ifndef _POSIX_VDISABLE
-#define _POSIX_VDISABLE ((unsigned char) 0)
+# define _POSIX_VDISABLE ((unsigned char) 0)
 #endif
 
 #define Control(c) ((c) & 0x1f)
 /* Canonical values for control characters. */
 #ifndef CINTR
-#define CINTR Control ('c')
+# define CINTR Control ('c')
 #endif
 #ifndef CQUIT
-#define CQUIT 28
+# define CQUIT 28
 #endif
 #ifndef CERASE
-#define CERASE 127
+# define CERASE 127
 #endif
 #ifndef CKILL
-#define CKILL Control ('u')
+# define CKILL Control ('u')
 #endif
 #ifndef CEOF
-#define CEOF Control ('d')
+# define CEOF Control ('d')
 #endif
 #ifndef CEOL
-#define CEOL _POSIX_VDISABLE
+# define CEOL _POSIX_VDISABLE
 #endif
 #ifndef CSTART
-#define CSTART Control ('q')
+# define CSTART Control ('q')
 #endif
 #ifndef CSTOP
-#define CSTOP Control ('s')
+# define CSTOP Control ('s')
 #endif
 #ifndef CSUSP
-#define CSUSP Control ('z')
+# define CSUSP Control ('z')
 #endif
 #if defined(VEOL2) && !defined(CEOL2)
-#define CEOL2 _POSIX_VDISABLE
+# define CEOL2 _POSIX_VDISABLE
+#endif
+/* ISC renamed swtch to susp for termios, but we'll accept either name.  */
+#if defined(VSUSP) && !defined(VSWTCH)
+# define VSWTCH VSUSP
+# define CSWTCH CSUSP
 #endif
 #if defined(VSWTCH) && !defined(CSWTCH)
-#define CSWTCH _POSIX_VDISABLE
+# define CSWTCH _POSIX_VDISABLE
 #endif
 
 /* SunOS 5.3 loses (^Z doesn't work) if `swtch' is the same as `susp'.
    So the default is to disable `swtch.'  */
 #if defined (__sparc__) && defined (__svr4__)
-#undef CSWTCH
-#define CSWTCH _POSIX_VDISABLE
+# undef CSWTCH
+# define CSWTCH _POSIX_VDISABLE
 #endif
 
 #if defined(VWERSE) && !defined (VWERASE)	/* AIX-3.2.5 */
-#define VWERASE VWERSE
+# define VWERASE VWERSE
 #endif
 #if defined(VDSUSP) && !defined (CDSUSP)
-#define CDSUSP Control ('y')
+# define CDSUSP Control ('y')
 #endif
 #if !defined(VREPRINT) && defined(VRPRNT)	/* Irix 4.0.5 */
-#define VREPRINT VRPRNT
+# define VREPRINT VRPRNT
 #endif
 #if defined(VREPRINT) && !defined(CRPRNT)
-#define CRPRNT Control ('r')
+# define CRPRNT Control ('r')
 #endif
 #if defined(VWERASE) && !defined(CWERASE)
-#define CWERASE Control ('w')
+# define CWERASE Control ('w')
 #endif
 #if defined(VLNEXT) && !defined(CLNEXT)
-#define CLNEXT Control ('v')
+# define CLNEXT Control ('v')
 #endif
 #if defined(VDISCARD) && !defined(VFLUSHO)
-#define VFLUSHO VDISCARD
+# define VFLUSHO VDISCARD
 #endif
 #if defined(VFLUSH) && !defined(VFLUSHO)	/* Ultrix 4.2 */
-#define VFLUSHO VFLUSH
+# define VFLUSHO VFLUSH
+#endif
+#if defined(CTLECH) && !defined(ECHOCTL)	/* Ultrix 4.3 */
+# define ECHOCTL CTLECH
+#endif
+#if defined(TCTLECH) && !defined(ECHOCTL)	/* Ultrix 4.2 */
+# define ECHOCTL TCTLECH
+#endif
+#if defined(CRTKIL) && !defined(ECHOKE)		/* Ultrix 4.2 and 4.3 */
+# define ECHOKE CRTKIL
 #endif
 #if defined(VFLUSHO) && !defined(CFLUSHO)
-#define CFLUSHO Control ('o')
+# define CFLUSHO Control ('o')
 #endif
 #if defined(VSTATUS) && !defined(CSTATUS)
-#define CSTATUS Control ('t')
+# define CSTATUS Control ('t')
 #endif
 
-static const char *visible ();
-static unsigned long baud_to_value ();
-static int recover_mode ();
-static int screen_columns ();
-static int set_mode ();
-static long integer_arg ();
-static speed_t string_to_baud ();
-static tcflag_t *mode_type_flag ();
-static void display_all ();
-static void display_changed ();
-static void display_recoverable ();
-static void display_settings ();
-static void display_speed ();
-static void display_window_size ();
-static void sane_mode ();
-static void set_control_char ();
-static void set_speed ();
-static void set_window_size ();
-
-void error ();
-
 /* Which speeds to set.  */
 enum speed_setting
   {
@@ -265,9 +265,9 @@ static struct mode_info mode_info[] =
   {"tab1", output, SANE_UNSET, TAB1, TABDLY},
   {"tab0", output, SANE_SET, TAB0, TABDLY},
 #else
-#ifdef OXTABS
+# ifdef OXTABS
   {"tab3", output, SANE_UNSET, OXTABS, 0},
-#endif
+# endif
 #endif
 #ifdef BSDLY
   {"bs1", output, SANE_UNSET, BS1, BSDLY},
@@ -391,6 +391,30 @@ static struct control_info control_info[] =
   {NULL, 0, 0}
 };
 
+static const char *visible __P ((unsigned int ch));
+static unsigned long baud_to_value __P ((speed_t speed));
+static int recover_mode __P ((char *arg, struct termios *mode));
+static int screen_columns __P ((void));
+static int set_mode __P ((struct mode_info *info, int reversed,
+			  struct termios *mode));
+static long integer_arg __P ((const char *s));
+static speed_t string_to_baud __P ((const char *arg));
+static tcflag_t *mode_type_flag __P ((enum mode_type type,
+				      struct termios *mode));
+static void display_all __P ((struct termios *mode));
+static void display_changed __P ((struct termios *mode));
+static void display_recoverable __P ((struct termios *mode));
+static void display_settings __P ((enum output_type output_type,
+				   struct termios *mode));
+static void display_speed __P ((struct termios *mode, int fancy));
+static void display_window_size __P ((int fancy));
+static void sane_mode __P ((struct termios *mode));
+static void set_control_char __P ((struct control_info *info, const char *arg,
+				   struct termios *mode));
+static void set_speed __P ((enum speed_setting type, const char *arg,
+			    struct termios *mode));
+static void set_window_size __P ((int rows, int cols));
+
 /* The width of the screen, for output wrapping. */
 static int max_col;
 
@@ -413,10 +437,10 @@ char *program_name;
 
 /* VARARGS */
 static void
-#ifdef __STDC__
+#if PROTOTYPES
 wrapf (const char *message,...)
 #else
-wrapf (message, va_alist)
+ wrapf (message, va_alist)
      const char *message;
      va_dcl
 #endif
@@ -444,20 +468,20 @@ wrapf (message, va_alist)
 }
 
 static void
-usage (status)
-     int status;
+usage (int status)
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("\
+      printf (_("\
 Usage: %s [SETTING]...\n\
   or:  %s OPTION\n\
-",
+"),
 	      program_name, program_name);
-      printf ("\
+      printf (_("\
+Print or change terminal characteristics.\n\
 \n\
   -a, --all       print all current settings in human-readable form\n\
   -g, --save      print all current settings in a stty-readable form\n\
@@ -466,8 +490,8 @@ Usage: %s [SETTING]...\n\
 \n\
 Optional - before SETTING indicates negation.  An * marks non-POSIX\n\
 settings.  The underlying system defines which settings are available.\n\
-");
-      printf ("\
+"));
+      printf (_("\
 \n\
 Special characters:\n\
 * dsusp CHAR    CHAR will send a terminal stop signal once input flushed\n\
@@ -485,8 +509,8 @@ Special characters:\n\
   susp CHAR     CHAR will send a terminal stop signal\n\
 * swtch CHAR    CHAR will switch to a different shell layer\n\
 * werase CHAR   CHAR will erase the last word typed\n\
-");
-      printf ("\
+"));
+      printf (_("\
 \n\
 Special settings:\n\
   N             set the input and output speeds to N bauds\n\
@@ -500,8 +524,8 @@ Special settings:\n\
 * size          print the number of rows and columns according to the kernel\n\
   speed         print the terminal speed\n\
   time N        with -icanon, set read timeout of N tenths of a second\n\
-");
-      printf ("\
+"));
+      printf (_("\
 \n\
 Control settings:\n\
   [-]clocal     disable modem control signals\n\
@@ -513,15 +537,15 @@ Control settings:\n\
   [-]hupcl      same as [-]hup\n\
   [-]parenb     generate parity bit in output and expect parity bit in input\n\
   [-]parodd     set odd parity (even with `-')\n\
-");
-      printf ("\
+"));
+      printf (_("\
 \n\
 Input settings:\n\
   [-]brkint     breaks cause an interrupt signal\n\
   [-]icrnl      translate carriage return to newline\n\
-  [-]ignbrk     ignore breaks\n\
+  [-]ignbrk     ignore break characters\n\
   [-]igncr      ignore carriage return\n\
-  [-]ignpar     ignore parity errors\n\
+  [-]ignpar     ignore characters with parity errors\n\
 * [-]imaxbel    beep and do not flush a full input buffer on a character\n\
   [-]inlcr      translate newline to carriage return\n\
   [-]inpck      enable input parity checking\n\
@@ -532,8 +556,8 @@ Input settings:\n\
   [-]ixon       enable XON/XOFF flow control\n\
   [-]parmrk     mark parity errors (with a 255-0-character sequence)\n\
   [-]tandem     same as [-]ixoff\n\
-");
-      printf ("\
+"));
+      printf (_("\
 \n\
 Output settings:\n\
 * bsN           backspace delay style, N in [0..1]\n\
@@ -552,8 +576,8 @@ Output settings:\n\
 * tabs          same as tab0\n\
 * -tabs         same as tab3\n\
 * vtN           vertical tab delay style, N in [0..1]\n\
-");
-      printf ("\
+"));
+      printf (_("\
 \n\
 Local settings:\n\
   [-]crterase   echo erase characters as backspace-space-backspace\n\
@@ -574,8 +598,8 @@ Local settings:\n\
 * [-]prterase   same as [-]echoprt\n\
 * [-]tostop     stop background jobs that try to write to the terminal\n\
 * [-]xcase      with icanon, escape with `\\' for uppercase characters\n\
-");
-      printf ("\
+"));
+      printf (_("\
 \n\
 Combination settings:\n\
 * [-]LCASE      same as [-]lcase\n\
@@ -606,27 +630,26 @@ Combination settings:\n\
                 -imaxbel -opost -isig -icanon -xcase min 1 time 0\n\
   -raw          same as cooked\n\
   sane          same as cread -ignbrk brkint -inlcr -igncr icrnl\n\
-                -ixoff -iucl -ixany imaxbel opost -olcuc -ocrnl onlcr\n\
+                -ixoff -iuclc -ixany imaxbel opost -olcuc -ocrnl onlcr\n\
                 -onocr -onlret -ofill -ofdel nl0 cr0 tab0 bs0 vt0 ff0\n\
                 isig icanon iexten echo echoe echok -echonl -noflsh\n\
                 -xcase -tostop -echoprt echoctl echoke, all special\n\
                 characters to their default values.\n\
-");
-      printf ("\
+"));
+      printf (_("\
 \n\
 Handle the tty line connected to standard input.  Without arguments,\n\
 prints baud rate, line discipline, and deviations from stty sane.  In\n\
 settings, CHAR is taken literally, or coded as in ^c, 0x37, 0177 or\n\
 127; special values ^- or undef used to disable special characters.\n\
-");
+"));
+      puts (_("\nReport bugs to sh-utils-bugs@gnu.ai.mit.edu"));
     }
   exit (status);
 }
 
-void
-main (argc, argv)
-     int argc;
-     char **argv;
+int
+main (int argc, char **argv)
 {
   struct termios mode;
   enum output_type output_type;
@@ -638,8 +661,11 @@ main (argc, argv)
   int k;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 
-  parse_long_options (argc, argv, "stty", version_string, usage);
+  parse_long_options (argc, argv, "stty", GNU_PACKAGE, VERSION, usage);
 
   output_type = changed;
   verbose_output = 0;
@@ -703,18 +729,18 @@ main (argc, argv)
   /* Specifying both -a and -g gets an error.  */
   if (verbose_output && recoverable_output)
     error (2, 0,
-	   "the options for verbose and stty-readable output styles are\n\
-\tmutually exclusive");
+	   _("the options for verbose and stty-readable output styles are\n\
+mutually exclusive"));
 
   /* Specifying any other arguments with -a or -g gets an error.  */
   if (argc - optind > 0 && (verbose_output || recoverable_output))
-    error (2, 0, "when specifying an output style, modes may not be set");
+    error (2, 0, _("when specifying an output style, modes may not be set"));
 
   /* Initialize to all zeroes so there is no risk memcmp will report a
      spurious difference in an uninitialized portion of the structure.  */
-  bzero (&mode, sizeof (mode));
+  memset (&mode, 0, sizeof (mode));
   if (tcgetattr (0, &mode))
-    error (1, errno, "standard input");
+    error (1, errno, _("standard input"));
 
   if (verbose_output || recoverable_output || argc == 1)
     {
@@ -749,7 +775,7 @@ main (argc, argv)
 	}
       if (match_found == 0 && reversed)
 	{
-	  error (0, 0, "invalid argument `%s'", --argv[k]);
+	  error (0, 0, _("invalid argument `%s'"), --argv[k]);
 	  usage (1);
 	}
       if (match_found == 0)
@@ -760,7 +786,7 @@ main (argc, argv)
 		{
 		  if (k == argc - 1)
 		    {
-		      error (0, 0, "missing argument to `%s'", argv[k]);
+		      error (0, 0, _("missing argument to `%s'"), argv[k]);
 		      usage (1);
 		    }
 		  match_found = 1;
@@ -777,7 +803,7 @@ main (argc, argv)
 	    {
 	      if (k == argc - 1)
 		{
-		  error (0, 0, "missing argument to `%s'", argv[k]);
+		  error (0, 0, _("missing argument to `%s'"), argv[k]);
 		  usage (1);
 		}
 	      ++k;
@@ -789,7 +815,7 @@ main (argc, argv)
 	    {
 	      if (k == argc - 1)
 		{
-		  error (0, 0, "missing argument to `%s'", argv[k]);
+		  error (0, 0, _("missing argument to `%s'"), argv[k]);
 		  usage (1);
 		}
 	      ++k;
@@ -802,7 +828,7 @@ main (argc, argv)
 	    {
 	      if (k == argc - 1)
 		{
-		  error (0, 0, "missing argument to `%s'", argv[k]);
+		  error (0, 0, _("missing argument to `%s'"), argv[k]);
 		  usage (1);
 		}
 	      ++k;
@@ -813,7 +839,7 @@ main (argc, argv)
 	    {
 	      if (k == argc - 1)
 		{
-		  error (0, 0, "missing argument to `%s'", argv[k]);
+		  error (0, 0, _("missing argument to `%s'"), argv[k]);
 		  usage (1);
 		}
 	      ++k;
@@ -831,7 +857,7 @@ main (argc, argv)
 	    {
 	      if (k == argc - 1)
 		{
-		  error (0, 0, "missing argument to `%s'", argv[k]);
+		  error (0, 0, _("missing argument to `%s'"), argv[k]);
 		  usage (1);
 		}
 	      ++k;
@@ -854,7 +880,7 @@ main (argc, argv)
 	    {
 	      if (recover_mode (argv[k], &mode) == 0)
 		{
-		  error (0, 0, "invalid argument `%s'", argv[k]);
+		  error (0, 0, _("invalid argument `%s'"), argv[k]);
 		  usage (1);
 		}
 	      require_set_attr = 1;
@@ -868,7 +894,7 @@ main (argc, argv)
       struct termios new_mode;
 
       if (tcsetattr (0, TCSADRAIN, &mode))
-	error (1, errno, "standard input");
+	error (1, errno, _("standard input"));
 
       /* POSIX (according to Zlotnick's book) tcsetattr returns zero if
 	 it performs *any* of the requested operations.  This means it
@@ -879,9 +905,9 @@ main (argc, argv)
 
       /* Initialize to all zeroes so there is no risk memcmp will report a
 	 spurious difference in an uninitialized portion of the structure.  */
-      bzero (&new_mode, sizeof (new_mode));
+      memset (&new_mode, 0, sizeof (new_mode));
       if (tcgetattr (0, &new_mode))
-	error (1, errno, "standard input");
+	error (1, errno, _("standard input"));
 
       /* Normally, one shouldn't use memcmp to compare structures that
 	 may have `holes' containing uninitialized data, but we have been
@@ -905,8 +931,16 @@ main (argc, argv)
 	  new_mode.c_cflag &= (~CIBAUD);
 	  if (speed_was_set || memcmp (&mode, &new_mode, sizeof (mode)) != 0)
 #endif
-	    error (1, 0,
-	      "standard input: unable to perform all requested operations");
+	    {
+	      size_t i;
+	      error (1, 0,
+	      _("standard input: unable to perform all requested operations"));
+	      printf (_("new_mode: mode\n"));
+	      for (i = 0; i < sizeof (new_mode); i++)
+		printf ("0x%02x: 0x%02x\n",
+			*(((unsigned char *) &new_mode) + i),
+			*(((unsigned char *) &mode) + i));
+	    }
 	}
     }
 
@@ -916,10 +950,7 @@ main (argc, argv)
 /* Return 0 if not applied because not reversible; otherwise return 1. */
 
 static int
-set_mode (info, reversed, mode)
-     struct mode_info *info;
-     int reversed;
-     struct termios *mode;
+set_mode (struct mode_info *info, int reversed, struct termios *mode)
 {
   tcflag_t *bitsp;
 
@@ -1061,7 +1092,7 @@ set_mode (info, reversed, mode)
 	    mode->c_oflag = (mode->c_oflag & ~TABDLY) | TAB0;
 	}
 #else
-#ifdef OXTABS
+# ifdef OXTABS
       else if (!strcmp (info->name, "tabs"))
 	{
 	  if (reversed)
@@ -1069,7 +1100,7 @@ set_mode (info, reversed, mode)
 	  else
 	    mode->c_oflag = mode->c_oflag & ~OXTABS;
 	}
-#endif
+# endif
 #endif
 #if defined(XCASE) && defined(IUCLC) && defined(OLCUC)
       else if (!strcmp (info->name, "lcase")
@@ -1125,10 +1156,8 @@ set_mode (info, reversed, mode)
 }
 
 static void
-set_control_char (info, arg, mode)
-     struct control_info *info;
-     char *arg;
-     struct termios *mode;
+set_control_char (struct control_info *info, const char *arg,
+		  struct termios *mode)
 {
   unsigned char value;
 
@@ -1151,10 +1180,7 @@ set_control_char (info, arg, mode)
 }
 
 static void
-set_speed (type, arg, mode)
-     enum speed_setting type;
-     char *arg;
-     struct termios *mode;
+set_speed (enum speed_setting type, const char *arg, struct termios *mode)
 {
   speed_t baud;
 
@@ -1167,33 +1193,23 @@ set_speed (type, arg, mode)
 
 #ifdef TIOCGWINSZ
 
-/* Get window size information.  First try getting the information
-   associated with standard output and if that fails, try standard input.
-   Return zero for success, non-zero if both ioctl's failed.  */
-
 static int
-get_win_size (win)
-     struct winsize *win;
+get_win_size (int fd, struct winsize *win)
 {
-  int err;
-
-  err = ioctl (1, TIOCGWINSZ, (char *) win);
-  if (err != 0)
-    err = ioctl (0, TIOCGWINSZ, (char *) win);
+  int err = ioctl (fd, TIOCGWINSZ, (char *) win);
   return err;
 }
 
 static void
-set_window_size (rows, cols)
-     int rows, cols;
+set_window_size (int rows, int cols)
 {
   struct winsize win;
 
-  if (get_win_size (&win))
+  if (get_win_size (STDIN_FILENO, &win))
     {
       if (errno != EINVAL)
-	error (1, errno, "standard input");
-      bzero (&win, sizeof (win));
+	error (1, errno, _("standard input"));
+      memset (&win, 0, sizeof (win));
     }
 
   if (rows >= 0)
@@ -1201,7 +1217,7 @@ set_window_size (rows, cols)
   if (cols >= 0)
     win.ws_col = cols;
 
-#ifdef TIOCSSIZE
+# ifdef TIOCSSIZE
   /* Alexander Dupuy <dupuy@cs.columbia.edu> wrote:
      The following code deals with a bug in the SunOS 4.x (and 3.x?) kernel.
      This comment from sys/ttold.h describes Sun's twisted logic - a better
@@ -1233,29 +1249,30 @@ set_window_size (rows, cols)
       win.ws_row = 1;
       win.ws_col = 1;
 
-      if (ioctl (0, TIOCSWINSZ, (char *) &win))
-	error (1, errno, "standard input");
+      if (ioctl (STDIN_FILENO, TIOCSWINSZ, (char *) &win))
+	error (1, errno, _("standard input"));
 
-      if (ioctl (0, TIOCSSIZE, (char *) &ttysz))
-	error (1, errno, "standard input");
+      if (ioctl (STDIN_FILENO, TIOCSSIZE, (char *) &ttysz))
+	error (1, errno, _("standard input"));
       return;
     }
-#endif
+# endif
 
-  if (ioctl (0, TIOCSWINSZ, (char *) &win))
-    error (1, errno, "standard input");
+  if (ioctl (STDIN_FILENO, TIOCSWINSZ, (char *) &win))
+    error (1, errno, _("standard input"));
 }
 
 static void
-display_window_size (fancy)
-     int fancy;
+display_window_size (int fancy)
 {
   struct winsize win;
 
-  if (get_win_size (&win))
+  if (get_win_size (STDIN_FILENO, &win))
     {
       if (errno != EINVAL)
-	error (1, errno, "standard input");
+	error (1, errno, _("standard input"));
+      if (!fancy)
+	error (1, 0, _("no size information for this device"));
     }
   else
     {
@@ -1268,30 +1285,28 @@ display_window_size (fancy)
 #endif
 
 static int
-screen_columns ()
+screen_columns (void)
 {
 #ifdef TIOCGWINSZ
   struct winsize win;
 
-  if (get_win_size (&win))
-    {
-      /* With Solaris 2.[123], this ioctl fails and errno is set to
-	 EINVAL for telnet (but not rlogin) sessions.  */
-      if (errno != EINVAL)
-	error (1, errno, "standard input");
-    }
-  else if (win.ws_col > 0)
+  /* With Solaris 2.[123], this ioctl fails and errno is set to
+     EINVAL for telnet (but not rlogin) sessions.
+     On ISC 3.0, it fails for the console and the serial port
+     (but it works for ptys).
+     It can also fail on any system when stdout isn't a tty.
+     In case of any failure, just use the default.  */
+  if (get_win_size (STDOUT_FILENO, &win) == 0 && win.ws_col > 0)
     return win.ws_col;
 #endif
+  /* FIXME: use xstrtol */
   if (getenv ("COLUMNS"))
     return atoi (getenv ("COLUMNS"));
   return 80;
 }
 
 static tcflag_t *
-mode_type_flag (type, mode)
-     enum mode_type type;
-     struct termios *mode;
+mode_type_flag (enum mode_type type, struct termios *mode)
 {
   switch (type)
     {
@@ -1316,9 +1331,7 @@ mode_type_flag (type, mode)
 }
 
 static void
-display_settings (output_type, mode)
-     enum output_type output_type;
-     struct termios *mode;
+display_settings (enum output_type output_type, struct termios *mode)
 {
   switch (output_type)
     {
@@ -1337,8 +1350,7 @@ display_settings (output_type, mode)
 }
 
 static void
-display_changed (mode)
-     struct termios *mode;
+display_changed (struct termios *mode)
 {
   int i;
   int empty_line;
@@ -1358,6 +1370,19 @@ display_changed (mode)
     {
       if (mode->c_cc[control_info[i].offset] == control_info[i].saneval)
 	continue;
+      /* If swtch is the same as susp, don't print both.  */
+#if VSWTCH == VSUSP
+      if (strcmp (control_info[i].name, "swtch") == 0)
+	continue;
+#endif
+      /* If eof uses the same slot as min, only print whichever applies.  */
+#if VEOF == VMIN
+      if ((mode->c_lflag & ICANON) == 0
+	  && (strcmp (control_info[i].name, "eof") == 0
+	      || strcmp(control_info[i].name, "eol") == 0))
+	continue;
+#endif
+
       empty_line = 0;
       wrapf ("%s = %s;", control_info[i].name,
 	     visible (mode->c_cc[control_info[i].offset]));
@@ -1409,8 +1434,7 @@ display_changed (mode)
 }
 
 static void
-display_all (mode)
-     struct termios *mode;
+display_all (struct termios *mode)
 {
   int i;
   tcflag_t *bitsp;
@@ -1429,10 +1453,27 @@ display_all (mode)
 
   for (i = 0; strcmp (control_info[i].name, "min"); ++i)
     {
+      /* If swtch is the same as susp, don't print both.  */
+#if VSWTCH == VSUSP
+      if (strcmp (control_info[i].name, "swtch") == 0)
+	continue;
+#endif
+      /* If eof uses the same slot as min, only print whichever applies.  */
+#if VEOF == VMIN
+      if ((mode->c_lflag & ICANON) == 0
+	  && (strcmp (control_info[i].name, "eof") == 0
+	      || strcmp(control_info[i].name, "eol") == 0))
+	continue;
+#endif
       wrapf ("%s = %s;", control_info[i].name,
 	     visible (mode->c_cc[control_info[i].offset]));
     }
-  wrapf ("min = %d; time = %d;\n", mode->c_cc[VMIN], mode->c_cc[VTIME]);
+#if VEOF == VMIN
+  if ((mode->c_lflag & ICANON) == 0)
+#endif
+    wrapf ("min = %d; time = %d;", mode->c_cc[VMIN], mode->c_cc[VTIME]);
+  if (current_col != 0)
+    putchar ('\n');
   current_col = 0;
 
   for (i = 0; mode_info[i].name != NULL; ++i)
@@ -1458,9 +1499,7 @@ display_all (mode)
 }
 
 static void
-display_speed (mode, fancy)
-     struct termios *mode;
-     int fancy;
+display_speed (struct termios *mode, int fancy)
 {
   if (cfgetispeed (mode) == 0 || cfgetispeed (mode) == cfgetospeed (mode))
     wrapf (fancy ? "speed %lu baud;" : "%lu\n",
@@ -1474,8 +1513,7 @@ display_speed (mode, fancy)
 }
 
 static void
-display_recoverable (mode)
-     struct termios *mode;
+display_recoverable (struct termios *mode)
 {
   int i;
 
@@ -1488,9 +1526,7 @@ display_recoverable (mode)
 }
 
 static int
-recover_mode (arg, mode)
-     char *arg;
-     struct termios *mode;
+recover_mode (char *arg, struct termios *mode)
 {
   int i, n;
   unsigned int chr;
@@ -1513,6 +1549,11 @@ recover_mode (arg, mode)
       mode->c_cc[i] = chr;
       arg += n;
     }
+
+  /* Fail if there are too many fields.  */
+  if (*arg != '\0')
+    return 0;
+
   return 1;
 }
 
@@ -1554,8 +1595,7 @@ struct speed_map speeds[] =
 };
 
 static speed_t
-string_to_baud (arg)
-     char *arg;
+string_to_baud (const char *arg)
 {
   int i;
 
@@ -1566,8 +1606,7 @@ string_to_baud (arg)
 }
 
 static unsigned long
-baud_to_value (speed)
-     speed_t speed;
+baud_to_value (speed_t speed)
 {
   int i;
 
@@ -1578,8 +1617,7 @@ baud_to_value (speed)
 }
 
 static void
-sane_mode (mode)
-     struct termios *mode;
+sane_mode (struct termios *mode)
 {
   int i;
   tcflag_t *bitsp;
@@ -1612,14 +1650,13 @@ sane_mode (mode)
 /* Adapted from `cat' by Torbjorn Granlund.  */
 
 static const char *
-visible (ch)
-     unsigned char ch;
+visible (unsigned int ch)
 {
   static char buf[10];
   char *bpout = buf;
 
   if (ch == _POSIX_VDISABLE)
-    return "<undef>";
+    return _("<undef>");
 
   if (ch >= 32)
     {
@@ -1665,45 +1702,12 @@ visible (ch)
 /* From `od' by Richard Stallman.  */
 
 static long
-integer_arg (s)
-     char *s;
+integer_arg (const char *s)
 {
   long value;
-  int radix = 10;
-  char *p = s;
-  int c;
-
-  if (*p != '0')
-    radix = 10;
-  else if (*++p == 'x')
+  if (xstrtol (s, NULL, 0, &value, "bB") != LONGINT_OK)
     {
-      radix = 16;
-      p++;
-    }
-  else
-    radix = 8;
-
-  value = 0;
-  while (((c = *p++) >= '0' && c <= '9')
-	 || (radix == 16 && (c & ~40) >= 'A' && (c & ~40) <= 'Z'))
-    {
-      value *= radix;
-      if (c >= '0' && c <= '9')
-	value += c - '0';
-      else
-	value += (c & ~40) - 'A';
-    }
-
-  if (c == 'b')
-    value *= 512;
-  else if (c == 'B')
-    value *= 1024;
-  else
-    p--;
-
-  if (*p)
-    {
-      error (0, 0, "invalid integer argument `%s'", s);
+      error (0, 0, _("invalid integer argument `%s'"), s);
       usage (1);
     }
   return value;
