@@ -2,7 +2,7 @@
 
 /* Modified to run with the GNU shell by bfox. */
 
-/* Copyright (C) 1987-1993, 1994 Free Software Foundation, Inc.
+/* Copyright (C) 1987-1993, 1994, 95, 1996 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -16,11 +16,11 @@
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.
 
-   You should have received a copy of the GNU General Public License along
-   with Bash; see the file COPYING.  If not, write to the Free Software
-   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-/* Define TEST_STANDALONE to get the /bin/test version.  Otherwise, you get 
+/* Define TEST_STANDALONE to get the /bin/test version.  Otherwise, you get
    the shell builtin version. */
 /* #define TEST_STANDALONE */
 
@@ -28,16 +28,16 @@
 #include <stdio.h>
 #include <sys/types.h>
 
+#define TEST_STANDALONE 1
+
 #if !defined (TEST_STANDALONE)
 #  include "shell.h"
 #  include "posixstat.h"
 #  include "filecntl.h"
 #else /* TEST_STANDALONE */
 #  include "system.h"
-#  include "version.h"
-#  include "safe-stat.h"
-#  include "safe-lstat.h"
 #  include "group-member.h"
+#  include "error.h"
 #  if !defined (S_IXUGO)
 #    define S_IXUGO 0111
 #  endif /* S_IXUGO */
@@ -66,10 +66,10 @@ extern int errno;
 #endif /* !STREQ */
 
 #if !defined (member)
-#  define member(c, s) ((c) ? (index ((s), (c)) ? 1 : 0) : 0)
+#  define member(c, s) ((c) ? (strchr ((s), (c)) ? 1 : 0) : 0)
 #endif /* !member */
 
-extern gid_t getgid (), getegid ();
+extern gid_t getegid ();
 extern uid_t geteuid ();
 
 #if !defined (R_OK)
@@ -109,18 +109,18 @@ static int pos;		/* The offset of the current argument in ARGV. */
 static int argc;	/* The number of arguments present in ARGV. */
 static char **argv;	/* The argument list. */
 
-static int unop ();
-static int binop ();
-static int unary_operator ();
-static int binary_operator ();
-static int two_arguments ();
-static int three_arguments ();
-static int posixtest ();
+static int unop __P ((int op));
+static int binop __P ((char *s));
+static int unary_operator __P ((void));
+static int binary_operator __P ((void));
+static int two_arguments __P ((void));
+static int three_arguments __P ((void));
+static int posixtest __P ((void));
 
-static int expr ();
-static int term ();
-static int and ();
-static int or ();
+static int expr __P ((void));
+static int term __P ((void));
+static int and __P ((void));
+static int or __P ((void));
 
 #if __GNUC__ >= 2 && defined (__GNUC_MINOR__) \
     && __GNUC_MINOR__ >= 5 && !defined (__STRICT_ANSI__)
@@ -129,12 +129,11 @@ static int or ();
 #define NO_RETURN_ATTRIBUTE /* empty */
 #endif
 
-static void test_syntax_error () NO_RETURN_ATTRIBUTE;
-static void beyond () NO_RETURN_ATTRIBUTE;
+static void test_syntax_error __P ((char *format, char *arg)) NO_RETURN_ATTRIBUTE;
+static void beyond __P ((void)) NO_RETURN_ATTRIBUTE;
 
 static void
-test_syntax_error (format, arg)
-     char *format, *arg;
+test_syntax_error (char *format, char *arg)
 {
   fprintf (stderr, "%s: ", argv[0]);
   fprintf (stderr, format, arg);
@@ -144,25 +143,21 @@ test_syntax_error (format, arg)
 
 /* A wrapper for stat () which disallows pathnames that are empty strings. */
 static int
-test_stat (path, finfo)
-     char *path;
-     struct stat *finfo;
+test_stat (char *path, struct stat *finfo)
 {
   if (*path == '\0')
     {
       errno = ENOENT;
       return (-1);
     }
-  return (SAFE_STAT (path, finfo));
+  return (stat (path, finfo));
 }
 
 /* Do the same thing access(2) does, but use the effective uid and gid,
-   and don't make the mistake of telling root that any file is
-   executable. */
+   and don't make the mistake of telling root that any file is executable.
+   But this loses when the containing filesystem is mounted e.g. read-only.  */
 static int
-eaccess (path, mode)
-     char *path;
-     int mode;
+eaccess (char *path, int mode)
 {
   struct stat st;
   static int euid = -1;
@@ -210,8 +205,7 @@ eaccess (path, mode)
 
 #if !defined (advance)
 static int
-advance (f)
-     int f;
+advance (int f)
 {
   ++pos;
 
@@ -233,27 +227,24 @@ advance (f)
  *	error condition)
  */
 static void
-beyond ()
+beyond (void)
 {
-  test_syntax_error ("argument expected\n", (char *)NULL);
+  test_syntax_error (_("argument expected\n"), NULL);
 }
 
 /* Syntax error for when an integer argument was expected, but
    something else was found. */
 static void
-integer_expected_error (pch)
-     char *pch;
+integer_expected_error (char *pch)
 {
-  test_syntax_error ("integer expression expected %s\n", pch);
+  test_syntax_error (_("integer expression expected %s\n"), pch);
 }
 
-/* Return non-zero if the characters pointed to by STRING constitute a
+/* Return nonzero if the characters pointed to by STRING constitute a
    valid number.  Stuff the converted number into RESULT if RESULT is
    a non-null pointer to a long. */
 static int
-isint (string, result)
-     register char *string;
-     long *result;
+isint (register char *string, long int *result)
 {
   int sign;
   long value;
@@ -308,11 +299,9 @@ isint (string, result)
 }
 
 /* Find the modification time of FILE, and stuff it into AGE, a pointer
-   to a long.  Return non-zero if successful, else zero. */
+   to a long.  Return nonzero if successful, else zero. */
 static int
-age_of (filename, age)
-     char *filename;
-     long *age;
+age_of (char *filename, long int *age)
 {
   struct stat finfo;
 
@@ -344,7 +333,7 @@ age_of (filename, age)
  *	positive and negative integers
  */
 static int
-term ()
+term (void)
 {
   int value;
 
@@ -364,13 +353,16 @@ term ()
       return (value ^ (term ()));
     }
 
-  /* A paren-bracketed argument. */  
+  /* A paren-bracketed argument. */
   if (argv[pos][0] == '(' && !argv[pos][1])
     {
       advance (1);
       value = expr ();
-      if (argv[pos][0] != ')' || argv[pos][1])
-	test_syntax_error ("')' expected, found %s\n", argv[pos]);
+      if (!argv[pos])
+	test_syntax_error (_("')' expected\n"), NULL);
+      else
+        if (argv[pos][0] != ')' || argv[pos][1])
+	  test_syntax_error (_("')' expected, found %s\n"), argv[pos]);
       advance (0);
       return (TRUE == (value));
     }
@@ -386,7 +378,7 @@ term ()
       if (unop (argv[pos][1]))
 	value = unary_operator ();
       else
-	test_syntax_error ("%s: unary operator expected\n", argv[pos]);
+	test_syntax_error (_("%s: unary operator expected\n"), argv[pos]);
     }
   else
     {
@@ -398,7 +390,7 @@ term ()
 }
 
 static int
-binary_operator ()
+binary_operator (void)
 {
   register int op;
   struct stat stat_buf, stat_spare;
@@ -413,7 +405,7 @@ binary_operator ()
 
       /* Make sure that OP is still a valid binary operator. */
       if ((op >= argc - 1) || (binop (argv[op]) == 0))
-	test_syntax_error ("%s: binary operator expected\n", argv[op]);
+	test_syntax_error (_("%s: binary operator expected\n"), argv[op]);
 
       advance (0);
     }
@@ -448,7 +440,7 @@ binary_operator ()
 	      else
 		{
 		  if (!isint (argv[op - 1], &l))
-		    integer_expected_error ("before -lt");
+		    integer_expected_error (_("before -lt"));
 		}
 
 	      if (r_is_l)
@@ -456,7 +448,7 @@ binary_operator ()
 	      else
 		{
 		  if (!isint (argv[op + 1], &r))
-		    integer_expected_error ("after -lt");
+		    integer_expected_error (_("after -lt"));
 		}
 	      pos += 3;
 	      return (TRUE == (l < r));
@@ -470,14 +462,14 @@ binary_operator ()
 	      else
 		{
 		  if (!isint (argv[op - 1], &l))
-		    integer_expected_error ("before -le");
+		    integer_expected_error (_("before -le"));
 		}
 	      if (r_is_l)
 		r = strlen (argv[op + 2]);
 	      else
 		{
 		  if (!isint (argv[op + 1], &r))
-		    integer_expected_error ("after -le");
+		    integer_expected_error (_("after -le"));
 		}
 	      pos += 3;
 	      return (TRUE == (l <= r));
@@ -493,14 +485,14 @@ binary_operator ()
 	      else
 		{
 		  if (!isint (argv[op - 1], &l))
-		    integer_expected_error ("before -gt");
+		    integer_expected_error (_("before -gt"));
 		}
 	      if (r_is_l)
 		r = strlen (argv[op + 2]);
 	      else
 		{
 		  if (!isint (argv[op + 1], &r))
-		    integer_expected_error ("after -gt");
+		    integer_expected_error (_("after -gt"));
 		}
 	      pos += 3;
 	      return (TRUE == (l > r));
@@ -514,14 +506,14 @@ binary_operator ()
 	      else
 		{
 		  if (!isint (argv[op - 1], &l))
-		    integer_expected_error ("before -ge");
+		    integer_expected_error (_("before -ge"));
 		}
 	      if (r_is_l)
 		r = strlen (argv[op + 2]);
 	      else
 		{
 		  if (!isint (argv[op + 1], &r))
-		    integer_expected_error ("after -ge");
+		    integer_expected_error (_("after -ge"));
 		}
 	      pos += 3;
 	      return (TRUE == (l >= r));
@@ -534,7 +526,7 @@ binary_operator ()
 	      /* nt - newer than */
 	      pos += 3;
 	      if (l_is_l || r_is_l)
-		test_syntax_error ("-nt does not accept -l\n", (char *)NULL);
+		test_syntax_error (_("-nt does not accept -l\n"), NULL);
 	      if (age_of (argv[op - 1], &l) && age_of (argv[op + 1], &r))
 		return (TRUE == (l > r));
 	      else
@@ -549,14 +541,14 @@ binary_operator ()
 	      else
 		{
 		  if (!isint (argv[op - 1], &l))
-		    integer_expected_error ("before -ne");
+		    integer_expected_error (_("before -ne"));
 		}
 	      if (r_is_l)
 		r = strlen (argv[op + 2]);
 	      else
 		{
 		  if (!isint (argv[op + 1], &r))
-		    integer_expected_error ("after -ne");
+		    integer_expected_error (_("after -ne"));
 		}
 	      pos += 3;
 	      return (TRUE == (l != r));
@@ -572,14 +564,14 @@ binary_operator ()
 	      else
 		{
 		  if (!isint (argv[op - 1], &l))
-		    integer_expected_error ("before -eq");
+		    integer_expected_error (_("before -eq"));
 		}
 	      if (r_is_l)
 		r = strlen (argv[op + 2]);
 	      else
 		{
 		  if (!isint (argv[op + 1], &r))
-		    integer_expected_error ("after -eq");
+		    integer_expected_error (_("after -eq"));
 		}
 	      pos += 3;
 	      return (TRUE == (l == r));
@@ -590,10 +582,10 @@ binary_operator ()
 	      /* ef - hard link? */
 	      pos += 3;
 	      if (l_is_l || r_is_l)
-		test_syntax_error ("-ef does not accept -l\n", (char *)NULL);
-	      if (SAFE_STAT (argv[op - 1], &stat_buf) < 0)
+		test_syntax_error (_("-ef does not accept -l\n"), NULL);
+	      if (stat (argv[op - 1], &stat_buf) < 0)
 		return (FALSE);
-	      if (SAFE_STAT (argv[op + 1], &stat_spare) < 0)
+	      if (stat (argv[op + 1], &stat_spare) < 0)
 		return (FALSE);
 	      return (TRUE ==
 		      (stat_buf.st_dev == stat_spare.st_dev &&
@@ -607,14 +599,14 @@ binary_operator ()
 	      /* ot - older than */
 	      pos += 3;
 	      if (l_is_l || r_is_l)
-		test_syntax_error ("-nt does not accept -l\n", (char *)NULL);
+		test_syntax_error (_("-nt does not accept -l\n"), NULL);
 	      if (age_of (argv[op - 1], &l) && age_of (argv[op + 1], &r))
 		return (TRUE == (l < r));
 	      return (FALSE);
 	    }
 	  break;
 	}
-      test_syntax_error ("unknown binary operator", argv[op]);
+      test_syntax_error (_("unknown binary operator"), argv[op]);
     }
 
   if (argv[op][0] == '=' && !argv[op][1])
@@ -636,7 +628,7 @@ binary_operator ()
 }
 
 static int
-unary_operator ()
+unary_operator (void)
 {
   long r, value;
   struct stat stat_buf;
@@ -756,7 +748,7 @@ unary_operator ()
 #else
       /* An empty filename is not a valid pathname. */
       if ((argv[pos - 1][0] == '\0') ||
-	  (SAFE_LSTAT (argv[pos - 1], &stat_buf) < 0))
+	  (lstat (argv[pos - 1], &stat_buf) < 0))
 	return (FALSE);
 
       return (TRUE == (S_ISLNK (stat_buf.st_mode)));
@@ -813,14 +805,14 @@ unary_operator ()
       return (TRUE == (argv[pos - 1][0] == '\0'));
     }
 }
-	
+
 /*
  * and:
  *	term
  *	term '-a' and
  */
 static int
-and ()
+and (void)
 {
   int value;
 
@@ -839,7 +831,7 @@ and ()
  *	and '-o' or
  */
 static int
-or ()
+or (void)
 {
   int value;
 
@@ -859,7 +851,7 @@ or ()
  *	or
  */
 static int
-expr ()
+expr (void)
 {
   if (pos >= argc)
     beyond ();
@@ -869,8 +861,7 @@ expr ()
 
 /* Return TRUE if S is one of the test command's binary operators. */
 static int
-binop (s)
-     char *s;
+binop (char *s)
 {
   return ((STREQ (s,   "=")) || (STREQ (s,  "!=")) || (STREQ (s, "-nt")) ||
 	  (STREQ (s, "-ot")) || (STREQ (s, "-ef")) || (STREQ (s, "-eq")) ||
@@ -878,16 +869,15 @@ binop (s)
 	  (STREQ (s, "-gt")) || (STREQ (s, "-ge")));
 }
 
-/* Return non-zero if OP is one of the test command's unary operators. */
+/* Return nonzero if OP is one of the test command's unary operators. */
 static int
-unop (op)
-     int op;
+unop (int op)
 {
   return (member (op, "abcdefgkLhprsStuwxOGnz"));
 }
 
 static int
-two_arguments ()
+two_arguments (void)
 {
   int value;
 
@@ -898,7 +888,7 @@ two_arguments ()
       if (unop (argv[pos][1]))
 	value = unary_operator ();
       else
-	test_syntax_error ("%s: unary operator expected\n", argv[pos]);
+	test_syntax_error (_("%s: unary operator expected\n"), argv[pos]);
     }
   else
     beyond ();
@@ -906,7 +896,7 @@ two_arguments ()
 }
 
 static int
-three_arguments ()
+three_arguments (void)
 {
   int value;
 
@@ -924,13 +914,13 @@ three_arguments ()
 	   (argv[pos][0] == '('))
     value = expr ();
   else
-    test_syntax_error ("%s: binary operator expected\n", argv[pos+1]);
+    test_syntax_error (_("%s: binary operator expected\n"), argv[pos+1]);
   return (value);
 }
 
 /* This is an implementation of a Posix.2 proposal by David Korn. */
 static int
-posixtest ()
+posixtest (void)
 {
   int value;
 
@@ -975,35 +965,35 @@ posixtest ()
 #include "long-options.h"
 
 static void
-usage (status)
-     int status;
+usage (int status)
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("\
+      printf (_("\
 Usage: %s EXPRESSION\n\
   or:  [ EXPRESSION ]\n\
   or:  %s OPTION\n\
-",
+"),
 	      program_name, program_name);
-      printf ("\
+      printf (_("\
+Exit with the status determined by EXPRESSION.\n\
 \n\
   --help      display this help and exit\n\
   --version   output version information and exit\n\
 \n\
 EXPRESSION is true or false and sets exit status.  It is one of:\n\
-");
-      printf ("\
+"));
+      printf (_("\
 \n\
   ( EXPRESSION )               EXPRESSION is true\n\
   ! EXPRESSION                 EXPRESSION is false\n\
   EXPRESSION1 -a EXPRESSION2   both EXPRESSION1 and EXPRESSION2 are true\n\
   EXPRESSION1 -o EXPRESSION2   either EXPRESSION1 or EXPRESSION2 is true\n\
 \n\
-  [-n] STRING          the length of STRING is non-zero\n\
+  [-n] STRING          the length of STRING is nonzero\n\
   -z STRING            the length of STRING is zero\n\
   STRING1 = STRING2    the strings are equal\n\
   STRING1 != STRING2   the strings are not equal\n\
@@ -1014,46 +1004,47 @@ EXPRESSION is true or false and sets exit status.  It is one of:\n\
   INTEGER1 -le INTEGER2   INTEGER1 is less than or equal to INTEGER2\n\
   INTEGER1 -lt INTEGER2   INTEGER1 is less than INTEGER2\n\
   INTEGER1 -ne INTEGER2   INTEGER1 is not equal to INTEGER2\n\
-");
-      printf ("\
+"));
+      printf (_("\
 \n\
   FILE1 -ef FILE2   FILE1 and FILE2 have the same device and inode numbers\n\
   FILE1 -nt FILE2   FILE1 is newer (modification date) than FILE2\n\
   FILE1 -ot FILE2   FILE1 is older than FILE2\n\
 \n\
-  -G FILE     FILE exists and is owned by the effective group ID\n\
-  -L FILE     FILE exists and is a symbolic link\n\
-  -O FILE     FILE exists and is owned by the effective user ID\n\
-  -S FILE     FILE exists and is a socket\n\
   -b FILE     FILE exists and is block special\n\
   -c FILE     FILE exists and is character special\n\
   -d FILE     FILE exists and is a directory\n\
   -e FILE     FILE exists\n\
   -f FILE     FILE exists and is a regular file\n\
   -g FILE     FILE exists and is set-group-ID\n\
+  -G FILE     FILE exists and is owned by the effective group ID\n\
   -k FILE     FILE exists and has its sticky bit set\n\
+  -L FILE     FILE exists and is a symbolic link\n\
+  -O FILE     FILE exists and is owned by the effective user ID\n\
   -p FILE     FILE exists and is a named pipe\n\
   -r FILE     FILE exists and is readable\n\
   -s FILE     FILE exists and has a size greater than zero\n\
-  -t          standard output is opened on a terminal\n\
-  -t FD       file descriptor FD is opened on a terminal\n\
+  -S FILE     FILE exists and is a socket\n\
+  -t [FD]     file descriptor FD (stdout by default) is opened on a terminal\n\
   -u FILE     FILE exists and its set-user-ID bit is set\n\
   -w FILE     FILE exists and is writable\n\
   -x FILE     FILE exists and is executable\n\
-");
-      printf ("\
+"));
+      printf (_("\
 \n\
-  --help      display this help and exit\n\
-  --version   output version information and exit\n\
-\n\
-Beware that parentheses need to be escaped by backslashes for shells.\n\
+Beware that parentheses need to be escaped (e.g., by backslashes) for shells.\n\
 INTEGER may also be -l STRING, which evaluates to the length of STRING.\n\
-");
+"));
+      puts (_("\nReport bugs to sh-utils-bugs@gnu.ai.mit.edu"));
     }
   exit (status);
 }
 #endif /* TEST_STANDALONE */
 
+#if !defined (TEST_STANDALONE)
+# define main test_command
+#endif
+
 /*
  * [:
  *	'[' expr ']'
@@ -1061,13 +1052,7 @@ INTEGER may also be -l STRING, which evaluates to the length of STRING.\n\
  *	test expr
  */
 int
-#if defined (TEST_STANDALONE)
-main (margc, margv)
-#else
-test_command (margc, margv)
-#endif /* !TEST_STANDALONE */
-     int margc;
-     char **margv;
+main (int margc, char **margv)
 {
   int value;
 
@@ -1080,13 +1065,18 @@ test_command (margc, margv)
     return (test_error_return);
 #else /* TEST_STANDALONE */
   program_name = margv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 #endif /* TEST_STANDALONE */
 
   argv = margv;
 
   if (margv[0] && strcmp (margv[0], "[") == 0)
     {
-      parse_long_options (argc, argv, COMMAND_NAME, version_string, usage);
+      /* Don't recognize --help or --version if POSIXLY_CORRECT is set.  */
+      if (getenv ("POSIXLY_CORRECT") == NULL)
+	parse_long_options (argc, argv, COMMAND_NAME, GNU_PACKAGE, VERSION, usage);
 
       --margc;
 
@@ -1094,7 +1084,7 @@ test_command (margc, margv)
 	test_exit (SHELL_BOOLEAN (FALSE));
 
       if (margv[margc] && strcmp (margv[margc], "]") != 0)
-	test_syntax_error ("missing `]'\n", (char *)NULL);
+	test_syntax_error (_("missing `]'\n"), NULL);
     }
 
   argc = margc;
@@ -1103,11 +1093,11 @@ test_command (margc, margv)
   if (pos >= argc)
     test_exit (SHELL_BOOLEAN (FALSE));
 
-  parse_long_options (argc, argv, COMMAND_NAME, version_string, usage);
+  parse_long_options (argc, argv, COMMAND_NAME, GNU_PACKAGE, VERSION, usage);
   value = posixtest ();
 
   if (pos != argc)
-    test_syntax_error ("too many arguments\n", (char *)NULL);
+    test_syntax_error (_("too many arguments\n"), NULL);
 
   test_exit (SHELL_BOOLEAN (value));
 }
