@@ -1,17 +1,20 @@
 #!/bin/sh
 
 usage="Usage: $0 [OPTION]...
+Exit successfully.
 
    --help      display this help and exit
-   --version   output version information and exit"
+   --version   output version information and exit
+
+Report bugs to sh-utils-bugs@gnu.ai.mit.edu"
 
 case $# in
   1 )
-    case "z${1}" in 
+    case "z${1}" in
       z--help )
 	 echo "$usage"; exit 0 ;;
       z--version )
-	 echo "true - @VERSION@"; exit 0 ;;
+	 echo "true (@GNU_PACKAGE@) @VERSION@"; exit 0 ;;
       * ) ;;
     esac
     ;;
